use crate::{Context, Handler};
use axum::{
    extract::{Path, State},
    http::StatusCode,
    Json,
};
use serde::{Deserialize, Serialize};
use verax::Asset;

#[derive(Deserialize)]
pub struct AccountId(verax::AccountId);

#[derive(Serialize)]
pub struct Balance {
    amount: String,
    cents: String,
    asset: Asset,
}

impl From<verax::Amount> for Balance {
    fn from(amount: verax::Amount) -> Self {
        Balance {
            amount: amount.to_string(),
            cents: amount.cents().to_string(),
            asset: amount.asset().clone(),
        }
    }
}

#[async_trait::async_trait]
impl Handler for AccountId {
    type Ok = Vec<Balance>;
    type Err = verax::Error;

    async fn handle(self, ctx: &Context) -> Result<Self::Ok, Self::Err> {
        Ok(ctx
            .ledger
            .get_balance(&self.0)
            .await?
            .into_iter()
            .map(|x| x.into())
            .collect())
    }
}

pub async fn handler(
    Path(account): Path<AccountId>,
    State(ctx): State<Context>,
) -> Result<Json<Vec<Balance>>, (StatusCode, String)> {
    Ok(Json(account.handle(&ctx).await.map_err(|err| {
        (
            StatusCode::BAD_REQUEST,
            serde_json::to_string(&err).unwrap_or_default(),
        )
    })?))
}
